@@ -18,20 +18,14 @@ class ASG(StackTemplate):
         self,
         stack,
         paco_ctx,
-        env_ctx,
         role_profile_arn,
         ec2_manager_user_data_script,
         ec2_manager_cache_id
     ):
         self.asg_config = asg_config = stack.resource
         asg_config_ref = asg_config.paco_ref_parts
-        self.env_ctx = env_ctx
         self.ec2_manager_cache_id = ec2_manager_cache_id
-        segment_stack = self.env_ctx.get_segment_stack(asg_config.segment)
-        super().__init__(
-            stack,
-            paco_ctx,
-        )
+        super().__init__(stack, paco_ctx)
         self.set_aws_name('ASG', self.resource_group_name, self.resource_name)
 
         # Troposphere
@@ -52,9 +46,7 @@ class ASG(StackTemplate):
         )
 
         # if the network for the ASG is disabled, only use an empty placeholder
-        env_region = get_parent_by_interface(asg_config, schemas.IEnvironmentRegion)
-        if not env_region.network.is_enabled():
-            self.set_template(template.to_yaml())
+        if not self.asg_config.env_region_obj.network.is_enabled():
             return
 
         security_group_list_param = self.create_cfn_ref_list_param(
@@ -137,7 +129,7 @@ class ASG(StackTemplate):
         )
         template.add_resource(launch_config_res)
 
-        subnet_list_ref = 'paco.ref {}'.format(segment_stack.template.config_ref)
+        subnet_list_ref = asg_config.env_region_obj.network.vpc.segments[asg_config.segment].paco_ref
         if asg_config.availability_zone == 'all':
             subnet_list_ref += '.subnet_id_list'
         else:
@@ -208,6 +200,36 @@ class ASG(StackTemplate):
             troposphere.autoscaling.Tag('Name', asg_dict['AutoScalingGroupName'], True)
         ]
 
+        # TODO: DNS: To be enabled once cftempaltes/iam_managed_policies.py
+        #            is ported to troposphere
+        # if len(asg_config.dns) > 0 and asg_config.is_enabled():
+        #     idx = 0
+        #     for dns_config in asg_config.dns:
+        #         if references.is_ref(dns_config.hosted_zone):
+        #             hosted_zone_value = dns_config.hosted_zone+'.id'
+        #         else:
+        #             hosted_zone_value = dns_config.hosted_zone
+        #         dns_hosted_zone_param = self.create_cfn_parameter(
+        #             param_type='String',
+        #             name=f'DNSHostedZone{idx}',
+        #             description=f'DNS Hosted Zone for index {idx}',
+        #             value=dns_value
+        #         )
+        #         asg_dict['Tags'].append(
+        #             troposphere.autoscaling.Tag(f'Paco-DNS-Hosted-Zone-{idx}', troposphere.Ref(dns_hosted_zone_param), True)
+        #         )
+        #         dns_domain_param = self.create_cfn_parameter(
+        #             param_type='String',
+        #             name=f'DNSDomain{idx}',
+        #             description=f'DNS Domain name for index {idx}',
+        #             value=dns_value
+        #         )
+        #         asg_dict['Tags'].append(
+        #             troposphere.autoscaling.Tag(f'Paco-DNS-Domain-{idx}', troposphere.Ref(dns_domain_param), True)
+        #         )
+
+        #         idx += 1
+
         # EIP
         if asg_config.eip != None and asg_config.is_enabled():
             if references.is_ref(asg_config.eip) == True:
@@ -280,35 +302,59 @@ class ASG(StackTemplate):
                 #)
                 #asg_dict['Tags'].append(ebs_device_tag)
 
+        # ECS Cluster Configuration
+        if asg_config.is_enabled() and asg_config.ecs != None:
+            ecs_cluster_name_param = self.create_cfn_parameter(
+                param_type='String',
+                name='ECSClusterName',
+                description='ECS Cluster Name',
+                value=asg_config.ecs.cluster + '.name'
+            )
+            asg_tag = troposphere.autoscaling.Tag(
+                'Paco-ECSCluster-Name',
+                troposphere.Ref(ecs_cluster_name_param),
+                True
+            )
+            asg_dict['Tags'].append(asg_tag)
+
         asg_res = troposphere.autoscaling.AutoScalingGroup.from_dict(
             'ASG',
             asg_dict
         )
         template.add_resource(asg_res)
         asg_res.DependsOn = launch_config_res
-        max_batch_size = 1
-        min_instances_in_service = 0
-        pause_time = 'PT0S'
-        wait_on_resource_signals = False
-        if asg_config.is_enabled() == True:
-            if asg_config.rolling_update_policy != None:
-                if asg_config.rolling_update_policy.is_enabled():
-                    max_batch_size = asg_config.rolling_update_policy.max_batch_size
-                    min_instances_in_service = asg_config.rolling_update_policy.min_instances_in_service
-                    pause_time = asg_config.rolling_update_policy.pause_time
-                    wait_on_resource_signals = asg_config.rolling_update_policy.wait_on_resource_signals
-            else:
-                max_batch_size = asg_config.update_policy_max_batch_size
-                min_instances_in_service = asg_config.update_policy_min_instances_in_service
 
-        asg_res.UpdatePolicy = troposphere.policies.UpdatePolicy(
-            AutoScalingRollingUpdate=troposphere.policies.AutoScalingRollingUpdate(
-                MaxBatchSize=max_batch_size,
-                MinInstancesInService=min_instances_in_service,
-                PauseTime=pause_time,
-                WaitOnResourceSignals=wait_on_resource_signals
+        # only create an UpdatePolicy if it is enabled
+        update_policy = asg_config.rolling_update_policy
+        if update_policy.enabled == True:
+            if update_policy.pause_time == '' and update_policy.wait_on_resource_signals == True:
+                # if wait_on_resource_signals is true the default pause time is 5 minutes
+                update_policy.pause_time = 'PT5M'
+            elif update_policy.pause_time == '':
+                update_policy.pause_time = 'PT0S'
+
+            min_instances_in_service_param = self.create_cfn_parameter(
+            param_type='String',
+            name='MinInstancesInService',
+            description='Roling update minimum instances to remain in service during update.',
+            value=update_policy.min_instances_in_service
+            )
+
+            # UpdatePolicy properties
+            asg_res.UpdatePolicy = troposphere.policies.UpdatePolicy(
+                AutoScalingRollingUpdate=troposphere.policies.AutoScalingRollingUpdate(
+                    MaxBatchSize=update_policy.max_batch_size,
+                    MinInstancesInService=troposphere.Ref(min_instances_in_service_param),
+                    PauseTime=update_policy.pause_time,
+                    WaitOnResourceSignals=update_policy.wait_on_resource_signals,
+                    SuspendProcesses=[
+                        'HealthCheck',
+                        'ReplaceUnhealthy',
+                        'AlarmNotification',
+                        'ScheduledActions'
+                    ]
+                )
             )
-        )
 
         self.create_output(
             title='ASGName',
