@@ -1,30 +1,36 @@
 from paco import utils
-from paco.core.yaml import YAML
 from paco.core.exception import StackException, PacoErrorCode, PacoException
-from paco.models import schemas, references
+from paco.models import schemas
 from paco.models.references import Reference
 from paco.models.locations import get_parent_by_interface
 from paco.stack import StackOutputParam, Stack
-from paco.utils import dict_of_dicts_merge, md5sum, big_join
+from paco.utils import md5sum, big_join
 import troposphere
+import re
 
 
-class StackOutputConfig():
-    def __init__(self, config_ref, key):
-        self.key = key
-        self.config_ref = config_ref
+single_quoted_cfn_regex = re.compile(r"(.*)'(!Ref\W+\S+)'(.*)")
 
-    def get_config_dict(self, stack):
-        conf_dict = current = {}
-        ref_part_list = self.config_ref.split('.')
-        for ref_part in ref_part_list:
-            current[ref_part] = {}
-            last_dict = current
-            current = current[ref_part]
 
-        last_dict[ref_part]['__name__'] = stack.get_outputs_value(self.key)
+def fix_yaml_tagged_string_quotes(body):
+    """
+    Remove single-quotes around CFN Tags from the generated CloudFormation YAML.
 
-        return conf_dict
+    Works around a bug in Troposphere .to_yaml(). This uses the cfn-flip library,
+    which in turn uses PyYAML to generate the YAML from Troposphere. This is a known bug
+    where a desired YAML such as:
+
+        - !Ref SecretArnOne
+
+    Is incorrectly wrapped in single-quotes:
+
+        - '!Ref SecretArnOne'
+
+    The fix requries cfn-flip using another YAML library that offers better control over the output:
+    https://github.com/awslabs/aws-cfn-template-flip/issues/48
+    """
+    new_body = single_quoted_cfn_regex.sub(r'\1\2\3', body)
+    return new_body
 
 class StackTemplate():
     """A CloudFormation template with access to a Stack object and a Project object."""
@@ -70,7 +76,10 @@ class StackTemplate():
     @property
     def body(self):
         if self._body == None:
-            self._body = self.template.to_yaml()
+            # generate YAML from Troposphere and perform replacement on '!Ref SomeParam'
+            self._body = fix_yaml_tagged_string_quotes(
+                self.template.to_yaml()
+            )
         return self._body
 
     @body.setter
@@ -131,6 +140,18 @@ class StackTemplate():
             troposphere.cloudformation.WaitConditionHandle(title="EmptyTemplatePlaceholder")
         )
 
+    # There is a case in ECS where we need to manually set ValueFrom in the
+    # ContainerDefinitions Secerts KeyValuePair list with a !Ref SomeParameterName
+    # value in a dict that will be used in .from_dict(). When troposphere outputs
+    # the resource to yaml, it will wrap the !Ref SomeParameterName in single
+    # quotes which causes CloudFormation to complain. To workaround this we are
+    # using !ManualTroposphereRef (to create uniquieness) and performing the
+    # following replace.
+    def fix_troposphere_manual_ref(self):
+        search = "'!ManualTroposphereRef "
+        replace = "!Ref '"
+        self.body = self.body.replace(search, replace)
+
     def paco_sub(self):
         "Perform paco.sub expressions with the substitution string"
         while True:
@@ -159,7 +180,6 @@ class StackTemplate():
                         message += "paco.sub '{}'\n".format(self.body[str_idx:end_str_idx])
                         raise StackException(PacoErrorCode.Unknown, message = message)
                     else:
-                        #print("break 2")
                         break
                 rep_1_idx = dollar_idx
                 rep_2_idx = self.body.find("}", rep_1_idx, end_str_idx)+1
@@ -206,7 +226,7 @@ class StackTemplate():
         !! DEPRECATED !! Use self.cfn_logical_id* methods
         Create a CloudFormation safe Logical name
         """
-
+        cf_name = ""
         sep_list = ['_','-','@','.']
         if sep != None:
             sep_list = [sep]
@@ -503,20 +523,6 @@ class StackTemplate():
             noecho,
         )
 
-    def register_stack_output_config(
-        self,
-        config_ref,
-        stack_output_key
-    ):
-        "Register stack output config"
-        if config_ref.startswith('paco.ref'):
-            raise PacoException(
-                PacoErrorCode.Unknown,
-                message='Registered stack output config reference must not start with paco.ref: ' + config_ref
-            )
-        stack_output_config = StackOutputConfig(config_ref, stack_output_key)
-        self.stack.stack_output_config_list.append(stack_output_config)
-
     def create_output(
         self,
         title=None,
@@ -540,9 +546,9 @@ class StackTemplate():
             )
         if type(ref) == list:
             for ref_item in ref:
-                self.register_stack_output_config(ref_item, title)
+                self.stack.register_stack_output_config(ref_item, title)
         elif type(ref) == str:
-            self.register_stack_output_config(ref, title)
+            self.stack.register_stack_output_config(ref, title)
 
     def gen_output(self, name, value):
         "Return name and value as a CFN YAML formatted string"
@@ -569,11 +575,11 @@ class StackTemplate():
 
         return role_name
 
-    def set_aws_name(self, template_name, first_id=None, second_id=None, third_id=None):
+    def set_aws_name(self, template_name, first_id=None, second_id=None, third_id=None, fourth_id=None):
         if isinstance(first_id, list):
             id_list = first_id
         else:
-            id_list = [first_id, second_id, third_id]
+            id_list = [first_id, second_id, third_id, fourth_id]
 
         # Exceptions: ApiGatewayRestApi | Lambda
         if self.paco_ctx.legacy_flag('cftemplate_aws_name_2019_09_17') == True and \
