@@ -1,18 +1,19 @@
-from awacs.aws import Action, Allow, PolicyDocument, Principal, Statement, Policy
+from awacs.aws import Action, Allow, Statement, Policy
+from troposphere import awslambda
+from paco.aws_api.awslambda.code import init_lambda_code
 from paco.cftemplates.cftemplates import StackTemplate
 from paco.cftemplates.eventsrule import create_event_rule_name
 from paco.models.locations import get_parent_by_interface
 from paco.models.loader import get_all_nodes
-from paco.models.references import resolve_ref, get_model_obj_from_ref, Reference
+from paco.models.references import get_model_obj_from_ref, Reference
 from paco.models import schemas
-from paco.utils import hash_smaller, prefixed_name
-from io import StringIO
-from enum import Enum
-import awacs.sdb
-import os
+from paco.stack import StackHooks
+from paco.utils import md5sum, prefixed_name
+from pathlib import Path
 import troposphere
 import troposphere.awslambda
 import troposphere.iam
+import zipfile
 
 
 class Lambda(StackTemplate):
@@ -35,6 +36,8 @@ class Lambda(StackTemplate):
         # if not enabled finish with only empty placeholder
         if not awslambda.is_enabled(): return
 
+        self.code_bucket_name = None
+
         # Parameters
         sdb_cache_param = self.create_cfn_parameter(
             name='EnableSDBCache',
@@ -115,11 +118,12 @@ class Lambda(StackTemplate):
 
         # Lambda VPC
         if awslambda.vpc_config != None:
-            vpc_security_group = self.create_cfn_parameter(
+            vpc_security_group = self.create_cfn_ref_list_param(
                 name='VpcSecurityGroupIdList',
                 param_type='List<AWS::EC2::SecurityGroup::Id>',
                 description='VPC Security Group Id List',
                 value=awslambda.vpc_config.security_groups,
+                ref_attribute='id',
             )
             # Segment SubnetList is a Segment stack Output based on availability zones
             segment_ref = awslambda.vpc_config.segments[0] + '.subnet_id_list'
@@ -134,7 +138,7 @@ class Lambda(StackTemplate):
                 'SubnetIds': troposphere.Ref(subnet_list_param),
             }
 
-        # Code object: S3 Bucket or inline ZipFile?
+        # Code object: S3 Bucket, inline ZipFile or deploy artifact?
         if awslambda.code.s3_bucket:
             if awslambda.code.s3_bucket.startswith('paco.ref '):
                 value = awslambda.code.s3_bucket + ".name"
@@ -157,36 +161,48 @@ class Lambda(StackTemplate):
                 'S3Key': troposphere.Ref(s3key_param),
             }
         else:
-            cfn_export_dict['Code'] = {
-                'ZipFile': awslambda.code.zipfile
-            }
+            zip_path = Path(awslambda.code.zipfile)
+            if zip_path.is_file():
+                if zipfile.is_zipfile(zip_path):
+                    cfn_s3_code = self.prepare_s3bucket_artifact(is_zip=True)
+                    cfn_export_dict['Code'] = cfn_s3_code
+                else:
+                    # Code is inline in CloudFormation template
+                    # ToDo: make this legacy or optional behaviour and create a Zip and upload to S3 Bucket
+                    cfn_export_dict['Code'] = {
+                        'ZipFile': zip_path.read_text()
+                    }
+            elif zip_path.is_dir():
+                cfn_s3_code = self.prepare_s3bucket_artifact()
+                cfn_export_dict['Code'] = cfn_s3_code
 
         # Environment variables
         var_export = {}
         if awslambda.environment != None and awslambda.environment.variables != None:
             for var in awslambda.environment.variables:
                 name = var.key.replace('_','')
-                self.create_cfn_parameter(
+                env_param = self.create_cfn_parameter(
                     name='EnvVar{}'.format(name),
                     param_type='String',
                     description='Env var for {}'.format(name),
                     value=var.value,
                 )
+                var_export[var.key] = troposphere.Ref(env_param)
             if awslambda.sdb_cache == True:
-                awslambda.add_environment_variable(
-                    'SDB_CACHE_DOMAIN', troposphere.Ref('LambdaSDBCacheDomain')
-                )
+                var_export['SDB_CACHE_DOMAIN'] = troposphere.Ref('LambdaSDBCacheDomain')
             if len(awslambda.log_group_names) > 0:
                 # Add PACO_LOG_GROUPS Environment Variable
-                awslambda.add_environment_variable(
-                    'PACO_LOG_GROUPS',
-                    ','.join(
-                        [prefixed_name(awslambda, loggroup_name, self.paco_ctx.legacy_flag) for loggroup_name in awslambda.log_group_names]
-                    )
+                paco_log_groups = [
+                    prefixed_name(awslambda, loggroup_name, self.paco_ctx.legacy_flag)
+                    for loggroup_name in awslambda.log_group_names
+                ]
+                paco_log_groups_param = self.create_cfn_parameter(
+                    name='EnvVariablePacoLogGroups',
+                    param_type='String',
+                    description='Env var for Paco Log Groups',
+                    value=','.join(paco_log_groups),
                 )
-            for var in awslambda.environment.variables:
-                var_export[var.key] = var.value
-
+                var_export['PACO_LOG_GROUPS'] = troposphere.Ref(paco_log_groups_param)
         cfn_export_dict['Environment'] = { 'Variables': var_export }
 
         # Lambda resource
@@ -196,6 +212,27 @@ class Lambda(StackTemplate):
         )
         self.template.add_resource(self.awslambda_resource)
 
+        # Published Version
+        if self.awslambda.edge != None and self.awslambda.edge.auto_publish_version != None:
+            version_name = self.create_cfn_logical_id(f"AutoPublishedVersion{self.awslambda.edge.auto_publish_version}")
+            version_resource = troposphere.awslambda.Version(
+                title=version_name,
+                FunctionName=troposphere.Ref(self.awslambda_resource),
+                Description="AutoPublished by Paco",
+            )
+            self.template.add_resource(version_resource)
+
+            self.create_output(
+                title='AutoPublishedFunctionVersionArn',
+                value=troposphere.Ref(version_resource),
+                ref=awslambda.paco_ref_parts + '.autoversion.arn',
+            )
+            self.create_output(
+                title='AutoPublishedFunctionVersion',
+                value=troposphere.GetAtt(version_resource, "Version"),
+                ref=awslambda.paco_ref_parts + '.autoversion.version',
+            )
+
         # SDB Cache with SDB Domain and SDB Domain Policy resources
         if awslambda.sdb_cache == True:
             sdb_domain_resource = troposphere.sdb.Domain(
@@ -262,10 +299,10 @@ class Lambda(StackTemplate):
             )
             idx += 1
 
-        # S3 Bucket notification permission(s)
+        # Lambda permissions for connected Paco resources
         app = get_parent_by_interface(awslambda, schemas.IApplication)
-        # detect if an S3Bucket resource in the application is configured to notify the Lambda
         for obj in get_all_nodes(app):
+            # S3 Bucket notification permission(s)
             if schemas.IS3Bucket.providedBy(obj):
                 seen = {}
                 if hasattr(obj, 'notifications'):
@@ -286,8 +323,7 @@ class Lambda(StackTemplate):
                                     )
                                     seen[s3_logical_name] = True
 
-        # Events Rule permission(s)
-        for obj in get_all_nodes(app):
+            # Events Rule permission(s)
             if schemas.IEventsRule.providedBy(obj):
                 seen = {}
                 for target in obj.targets:
@@ -319,6 +355,30 @@ class Lambda(StackTemplate):
                             )
                             seen[eventsrule_logical_name] = True
 
+            # IoT Analytics permission(s)
+            if schemas.IIoTAnalyticsPipeline.providedBy(obj):
+                seen = {}
+                for activity in obj.pipeline_activities.values():
+                    if activity.activity_type == 'lambda':
+                        target_ref = Reference(activity.function)
+                        target_ref.set_account_name(account_ctx.get_name())
+                        target_ref.set_region(aws_region)
+                        lambda_ref = Reference(awslambda.paco_ref)
+                        if target_ref.raw == lambda_ref.raw:
+                            # yes, the IoT Analytics Lambda Activity has a ref to this Lambda
+                            group = get_parent_by_interface(obj, schemas.IResourceGroup)
+                            iotap_logical_name = self.gen_cf_logical_name(group.name + obj.name, '_')
+                            if iotap_logical_name not in seen:
+                                rule_name = create_event_rule_name(obj)
+                                troposphere.awslambda.Permission(
+                                    title='IoTAnalyticsPipeline' + iotap_logical_name,
+                                    template=self.template,
+                                    Action="lambda:InvokeFunction",
+                                    FunctionName=troposphere.GetAtt(self.awslambda_resource, 'Arn'),
+                                    Principal='iotanalytics.amazonaws.com',
+                                )
+                                seen[iotap_logical_name] = True
+
         # Log group(s)
         loggroup_function_name = troposphere.Join(
             '', [
@@ -407,6 +467,85 @@ class Lambda(StackTemplate):
             ref=awslambda.paco_ref_parts + '.arn',
         )
 
+    def prepare_s3bucket_artifact(self, is_zip=False):
+        """
+        Add a Hook which will prepare the Lambda Code artifact and upload to an S3 Bucket if it
+        doesn't already exist.
+
+        Add Parameters for CodeS3Bucket and CodeS3Key. The Parameter values are placeholder values that
+        will be updated by the hook.
+        """
+        ignore_changes = True
+        # always create an artifact on CREATE
+        stack_hooks = StackHooks()
+        stack_hooks.add(
+            name='PrepLambdaCodeArtifactToS3Bucket',
+            stack_action=['create',],
+            stack_timing='pre',
+            hook_method=self.prepare_s3bucket_artifact_hook,
+            cache_method=self.prepare_s3bucket_artifact_cache,
+            hook_arg=is_zip,
+        )
+        self.stack.add_hooks(stack_hooks)
+
+        # only UPDATE an artifact if --auto-publish-code is true
+        if self.paco_ctx.auto_publish_code == True:
+            ignore_changes = False
+            stack_hooks = StackHooks()
+            stack_hooks.add(
+                name='PrepLambdaCodeArtifactToS3Bucket',
+                stack_action=['update'],
+                stack_timing='pre',
+                hook_method=self.prepare_s3bucket_artifact_hook,
+                cache_method=self.prepare_s3bucket_artifact_cache,
+                hook_arg=is_zip,
+            )
+            self.stack.add_hooks(stack_hooks)
+
+        self.s3bucket_param = self.create_cfn_parameter(
+            name='CodeS3Bucket',
+            description="S3 Bucket for the Lambda Code artifact",
+            param_type='String',
+            value='',
+            ignore_changes=ignore_changes,
+        )
+        self.s3key_param = self.create_cfn_parameter(
+            name='CodeS3Key',
+            description="S3 Key for the Lambda Code artifact",
+            param_type='String',
+            value='',
+            ignore_changes=ignore_changes,
+        )
+        cfn_s3_code = {
+            'S3Bucket': troposphere.Ref(self.s3bucket_param),
+            'S3Key': troposphere.Ref(self.s3key_param),
+        }
+        return cfn_s3_code
+
+    def prepare_s3bucket_artifact_hook(self, hook, is_zip):
+        if self.code_bucket_name == None:
+            self.prepare_s3bucket_artifact_cache(hook, is_zip)
+        self.awslambda.stack.set_parameter(
+            'CodeS3Bucket',
+            self.code_bucket_name,
+            ignore_changes=False
+        )
+        self.awslambda.stack.set_parameter(
+            'CodeS3Key',
+            self.code_artifact_name,
+            ignore_changes=False
+        )
+
+    def prepare_s3bucket_artifact_cache(self, hook, is_zip):
+        self.code_bucket_name, self.code_artifact_name, md5_hash = init_lambda_code(
+            self.paco_ctx.paco_buckets,
+            self.stack.resource,
+            self.awslambda.code.zipfile,
+            self.stack.account_ctx,
+            self.stack.aws_region,
+            is_zip=is_zip,
+        )
+        return md5_hash
 
     def add_log_group(self, loggroup_name, logical_name=None):
         "Add a LogGroup resource to the template"
@@ -429,7 +568,7 @@ class Lambda(StackTemplate):
         self.template.add_resource(loggroup_resource)
 
         # LogGroup Output
-        self.register_stack_output_config(
+        self.stack.register_stack_output_config(
             '{}.log_groups.{}.arn'.format(self.awslambda.paco_ref_parts, logical_name), loggroup_logical_id + 'Arn'
         )
         loggroup_output = troposphere.Output(
@@ -438,3 +577,4 @@ class Lambda(StackTemplate):
         )
         self.template.add_output(loggroup_output)
         return loggroup_resource
+
