@@ -1,27 +1,25 @@
+from paco.core.exception import StackException
+from paco.core.exception import PacoErrorCode, MissingAccountId, InvalidAccountName
+from paco.models import vocabulary
+from paco.models.references import Reference
+from paco.models.exceptions import InvalidPacoProjectFile
+from paco.models import references
+from paco.models import load_project_from_yaml
+from paco.models.references import get_model_obj_from_ref
+from paco.core.yaml import read_yaml_file
+from paco.config.interfaces import IAccountContext
+from paco.config.paco_buckets import PacoBuckets
+from shutil import copyfile
+from deepdiff import DeepDiff
+from zope.interface import implementer
 import paco.config.aws_credentials
 import paco.core.log
 import paco.controllers
 import paco.models.services
 import os, sys, re
 import pathlib
-import pkg_resources
-import ruamel.yaml
-from paco.core.exception import StackException
-from paco.core.exception import PacoErrorCode, MissingAccountId, InvalidAccountName
-from paco.models import vocabulary
-from paco.models.references import Reference
-from paco.models import references
-from paco.models import load_project_from_yaml
-from paco.core.yaml import read_yaml_file
-from shutil import copyfile
-
-# deepdiff turns on Deprecation warnings, we need to turn them back off
-# again right after import, otherwise 3rd libs spam dep warnings all over the place
-from deepdiff import DeepDiff
-import warnings
-warnings.simplefilter("ignore")
-
 
+@implementer(IAccountContext)
 class AccountContext(object):
     "Manages the credentials and connection to an AWS Account"
 
@@ -29,7 +27,7 @@ class AccountContext(object):
         self,
         paco_ctx,
         name,
-        mfa_account=None
+        mfa_account=None,
     ):
         self.name = name
         self.client_cache = {}
@@ -81,13 +79,16 @@ Add this manually or run `paco provision accounts` for this project.
     def get_name(self):
         return self.name
 
-    def gen_ref(self):
-        return 'paco.ref accounts.%s' % (self.get_name())
+    @property
+    def paco_ref(self):
+        return f'paco.ref accounts.{self.name}'
 
     def get_temporary_credentials(self):
         return self.aws_session.get_temporary_credentials()
 
     def get_mfa_session(self, admin_creds):
+        if self.paco_ctx.skip_account_ctx:
+            return None
         if self.aws_session == None:
             self.aws_session = paco.config.aws_credentials.PacoSTS(
                 self,
@@ -103,6 +104,8 @@ Add this manually or run `paco provision accounts` for this project.
         return self.aws_session.get_temporary_session()
 
     def get_session(self, force=False):
+        if self.paco_ctx.skip_account_ctx:
+            return None
         if self.aws_session == None:
             self.aws_session = paco.config.aws_credentials.PacoSTS(
                     self,
@@ -117,9 +120,12 @@ Add this manually or run `paco provision accounts` for this project.
             )
         if self.temp_aws_session == None or force == True:
             self.temp_aws_session = self.aws_session.get_temporary_session()
-
         return self.temp_aws_session
 
+    @property
+    def model_obj(self):
+        return self.config
+
     @property
     def id(self):
         return self.config.account_id
@@ -178,6 +184,7 @@ def print_diff_object(diff_obj, diff_obj_key):
     if diff_obj_key not in diff_obj.keys():
         return
     for root_change in diff_obj[diff_obj_key]:
+        change_t = None
         node_str = '.'.join(getFromSquareBrackets(root_change.path()))
         if diff_obj_key.endswith('_removed'):
             change_t = root_change.t1
@@ -200,12 +207,13 @@ def print_diff_object(diff_obj, diff_obj_key):
         print('')
 
 def create_log_col(col='', col_size=0, message_len=0, wrap_text=False):
+    "Create a formattedtext column"
     if col == '' or col == None:
-        return ' '*col_size
-    message_spc = ' '*message_len
+        return ' ' * col_size
+    message_spc = ' ' * message_len
 
     if col.find('\n') != -1:
-        message = col.replace('\n', '\n'+message_spc)
+        message = col.replace('\n', '\n' + message_spc)
     elif wrap_text == True and len(col) > col_size:
         pos = col_size
         while pos < len(col):
@@ -213,7 +221,9 @@ def create_log_col(col='', col_size=0, message_len=0, wrap_text=False):
             pos += message_len + col_size
         message = col
     else:
-        message = '{}{} '.format(col[:col_size], ' '*(col_size-len(col[:col_size])))
+        message = '{}{} '.format(
+            col[:col_size], ' ' * (col_size - len(col[:col_size]))
+        )
     return message
 
 
@@ -223,7 +233,10 @@ class PacoContext(object):
     """
 
     def __init__(self, home=None):
-        self.home = home
+        if home != None:
+            self.home = pathlib.Path(home)
+        else:
+            self.home = home
         # CLI Flags
         self.verbose = False
         self.nocache = False
@@ -233,15 +246,28 @@ class PacoContext(object):
         self.paco_path = os.getcwd()
         self.aws_name = "Paco"
         self.controllers = {}
-        self.services = {}
+        self.service_controllers = {}
         self.accounts = {}
         self.logger = paco.core.log.get_paco_logger()
         self.project = None
         self.master_account = None
         self.command = None
+        self.config_scope = None
         self.disable_validation = False
+        self.paco_buckets = None
+        self.skip_account_ctx = False
+        self.auto_publish_code = False
 
     def get_account_context(self, account_ref=None, account_name=None, netenv_ref=None):
+        """
+        Get an AccountContext for an AWS Account. Will return an existing object if an AccountContext
+        has already been created.
+
+        AccountContext can be specified in three ways:
+          account_ref: 'paco.ref accounts.dev'
+          account_name: 'dev'
+          netenv_ref: 'paco.ref netenv.mynet.dev.us-west-2.applications.myapp' # The 'dev' env is in the dev account
+        """
         if account_ref != None:
             ref = Reference(account_ref)
             account_name = ref.parts[1]
@@ -251,7 +277,7 @@ class PacoContext(object):
             account_ref = self.get_ref(account_ref)
             return self.get_account_context(account_ref=account_ref)
         elif account_name == None:
-            raise StackException(PacoErrorCode.Unknown, message = "get_account_context was only passed None: Not enough context to get account.")
+            raise InvalidAccountName("Get AccountContext failed. Must specify a valid account name")
 
         if account_name in self.accounts:
             return self.accounts[account_name]
@@ -259,7 +285,7 @@ class PacoContext(object):
         account_ctx = AccountContext(
             paco_ctx=self,
             name=account_name,
-            mfa_account=self.master_account
+            mfa_account=self.master_account,
         )
         self.accounts[account_name] = account_ctx
 
@@ -282,9 +308,10 @@ This directory contains several sub-directories that Paco uses:
     build/
     outputs/
     applied/
+    describe/
 
 """
-        return pathlib.Path(self.home + os.sep + '.paco-work')
+        return self.home / '.paco-work'
 
     @property
     def outputs_path(self):
@@ -298,10 +325,23 @@ This directory contains several sub-directories that Paco uses:
 
     @property
     def build_path(self):
-        "Return the path to the Paco applied directory"
+        "Return the path to the Paco build directory"
         return self.paco_work_path / 'build'
 
-    def load_project(self, project_init=False, project_only=False, master_only=False):
+    @property
+    def describe_path(self):
+        "Return the path to the Paco describe directory"
+        return self.paco_work_path / 'describe'
+
+    def load_project(
+        self,
+        project_init=False,
+        project_only=False,
+        master_only=False,
+        config_scope=None,
+        command_name=None,
+        validate_local_paths=True,
+    ):
         "Load a Paco Project from YAML, initialize settings and controllers, and load Service plug-ins."
         self.project_folder = self.home
         if project_init == True:
@@ -309,47 +349,74 @@ This directory contains several sub-directories that Paco uses:
 
         # Load the model from YAML
         print("Loading Paco project: %s" % (self.home))
-        self.project = load_project_from_yaml(self.project_folder, None)
+        self.project = load_project_from_yaml(
+            self.project_folder,
+            warn=self.warn,
+            validate_local_paths=validate_local_paths,
+        )
+        self.paco_buckets = PacoBuckets(self.project)
+        if self.verbose:
+            print("Finished loading.")
         if project_only == True:
             return
 
-        # Settings
+        # Locate a model object and summarize it
+        # init commands do not have a config_scope
+        # ToDo: the 'accounts' scope does resolve properly
+        model_obj = None
+        if self.config_scope not in [None, 'accounts']:
+            paco_ref = 'paco.ref {}'.format(self.config_scope)
+            model_obj = get_model_obj_from_ref(paco_ref, self.project)
+            if self.verbose:
+                print('Object selected to {}:'.format(self.command))
+                print('  Name: {}'.format(
+                    getattr(model_obj, 'name', 'unnamed')
+                ))
+                print('  Type: {}'.format(model_obj.__class__.__name__))
+                if getattr(model_obj, 'title', None):
+                    print('  Title: {}'.format(model_obj.title))
+                if hasattr(model_obj, 'paco_ref_parts'):
+                    print('  Reference: {}'.format(model_obj.paco_ref_parts))
+                print()
+
+            # Check Notifications and warn about Alarms without any notifications
+            if self.warn:
+                self.check_notification_config()
+
+        # AWS Credentials with the master account
         self.master_account = AccountContext(
             paco_ctx=self,
             name='master',
             mfa_account=None
         )
+
+        # Settings
         os.environ['AWS_DEFAULT_REGION'] = self.project['credentials'].aws_default_region
-        if master_only:
+        if master_only or self.config_scope == 'accounts':
             return
 
-        # Initialize Controllers so they can initialize their
-        # resolve_ref_obj's to allow reference lookups
+        # Initialize Controllers so they can set their resolve_ref_obj's for reference lookups
         self.get_controller('Route53')
         self.get_controller('CodeCommit')
-        self.get_controller('S3').init({'name': 'buckets'})
+        self.get_controller('S3')
         self.get_controller('SNSTopics')
+        self.get_controller('SNS')
 
         # Load the Service plug-ins
-        service_plugins = paco.models.services.list_service_plugins()
-        for plugin_name, plugin_module in service_plugins.items():
-            try:
-                self.project['service'][plugin_name.lower()]
-            except KeyError:
-                # ignore if no config files for a registered service
-                self.log_action_col("Skipping", 'Service', plugin_name)
-                continue
+        service_plugins = paco.models.services.list_enabled_services(self.home)
+        for service_info in service_plugins.values():
+            service_config = self.project['service'][service_info['name']]
+            self.log_section_start("Init", service_config)
+            service_controller = service_info['module'].get_service_controller(self, service_config)
+            service_controller.init(None, model_obj)
+            self.service_controllers[service_info['name']] = service_controller
 
-            self.log_action_col('Init', 'Service', plugin_name)
-            service = plugin_module.instantiate_class(self, self.project['service'][plugin_name.lower()])
-            service.init(None)
-            self.services[plugin_name.lower()] = service
-            self.log_action_col('Init', 'Service', plugin_name, 'Completed')
-
-    def get_controller(self, controller_type, command=None, model_obj=None):
+    def get_controller(self, controller_type, command=None, model_obj=None, model_paco_ref=None):
         """Gets a controller by name and calls .init() on it with any controller args"""
         controller_type = controller_type.lower()
         controller = None
+        if model_obj == None and model_paco_ref != None:
+            model_obj = references.get_model_obj_from_ref(model_paco_ref, self.project)
         if controller_type != 'service':
             if controller_type in self.controllers:
                 controller = self.controllers[controller_type]
@@ -357,11 +424,11 @@ This directory contains several sub-directories that Paco uses:
                 controller = paco.controllers.klass[controller_type](self)
                 self.controllers[controller_type] = controller
         else:
-            service_name = model_obj.paco_ref_list[1]
-            if service_name.lower() not in self.services:
+            service_name = model_obj.paco_ref_list[1].lower()
+            if service_name not in self.service_controllers:
                 message = "Could not find Service: {}".format(service_name)
                 raise StackException(PacoErrorCode.Unknown, message = message)
-            controller = self.services[service_name.lower()]
+            controller = self.service_controllers[service_name]
 
         controller.init(command, model_obj)
         return controller
@@ -396,6 +463,33 @@ This directory contains several sub-directories that Paco uses:
             account_ctx=account_ctx
         )
 
+    def check_notification_config(self):
+        """Detect misconfigured alarm notification situations.
+        This happens after both MonitorConfig and NetworkEnvironments have loaded.
+        """
+        if 'snstopics' in self.project['resource']:
+            for app in self.project.get_all_applications():
+                if app.is_enabled():
+                    for alarm_info in app.list_alarm_info():
+                        alarm = alarm_info['alarm']
+                        # warn on alarms with no subscriptions
+                        if len(alarm.notification_groups) == 0:
+                            print("WARNING: Alarm {} for app {} does not have any notifications.".format(
+                                alarm.name,
+                                app.name
+                            ))
+                        # alarms with groups that do not exist
+                        region = self.project.active_regions[0] # regions are all the same, just choose the first
+                        for groupname in alarm.notification_groups:
+                            if groupname not in self.project['resource']['snstopics'][region]:
+                                raise InvalidPacoProjectFile(
+                                    "Alarm {} for app {} notifies to group '{}' which does belong in Notification service group names.".format(
+                                        alarm.name,
+                                        app.name,
+                                        groupname
+                                    )
+                                )
+
     def confirm_yaml_changes(self, model_obj):
         """Confirm changes made to the Paco Project YAML from the last run"""
         if self.disable_validation == True:
@@ -504,6 +598,24 @@ This directory contains several sub-directories that Paco uses:
         applied_file_path, new_file_path = self.init_model_obj_store(model_obj)
         copyfile(new_file_path, applied_file_path)
 
+    def log_section_start(self, action, obj):
+        "Log start with a bar header"
+        if not self.verbose:
+            return
+        print("=== {}: {} ===".format(action, obj.paco_ref_parts))
+
+    def log_start(self, action, obj):
+        "Log start with a table header"
+        if not self.verbose:
+            return
+        print("> {}: start: {}".format(action, obj.paco_ref_parts))
+
+    def log_finish(self, action, obj):
+        "Log Init finish for a controller"
+        if not self.verbose:
+            return
+        print("< {}: finish: {}".format(action, obj.paco_ref_parts))
+
     def log_action_col(
         self,
         col_1,
@@ -513,11 +625,13 @@ This directory contains several sub-directories that Paco uses:
         return_it=False,
         enabled=True,
         col_1_size=10,
-        col_2_size=11,
-        col_3_size=15,
-        col_4_size=None
+        col_2_size=19, # Resource type, longest is "ElasticSearchDomain"
+        col_3_size=23,
+        col_4_size=None,
+        use_bars=True
     ):
-        # Skip verbose messages unless -v verbose is flagged
+        "Log an action in columns"
+        # Silence Init and Skipping messages unless in verbose mode
         if not self.verbose:
             if col_1 == 'Init':
                 return
@@ -525,7 +639,6 @@ This directory contains several sub-directories that Paco uses:
                 return
         if col_2 == '': col_2 = None
         if col_3 == '': col_3 = None
-        if col_4 == '': col_4 = None
         if col_4 != None and col_4_size == None: col_4_size = len(col_4)
 
         if enabled == False:
@@ -548,13 +661,17 @@ This directory contains several sub-directories that Paco uses:
         if col_4 != None:
             col_4_wrap_text = True
 
-        message = create_log_col(col_1, col_1_size, 0)
+        if use_bars:
+            bars = '| '
+        else:
+            bars = ''
+        message = bars + create_log_col(col_1, col_1_size, 0)
         if col_2 != None:
-            message += create_log_col(col_2, col_2_size, len(message), col_2_wrap_text)
+            message += bars + create_log_col(col_2, col_2_size, len(message), col_2_wrap_text)
             if col_3 != None:
-                message += create_log_col(col_3, col_3_size, len(message), col_3_wrap_text)
+                message += bars + create_log_col(col_3, col_3_size, len(message), col_3_wrap_text)
                 if col_4 != None:
-                    message += create_log_col(col_4, col_4_size, len(message), col_4_wrap_text)
+                    message += bars + create_log_col(col_4, col_4_size, len(message), col_4_wrap_text)
         if return_it == True:
             return message+'\n'
         print(message)
