@@ -1,21 +1,15 @@
 from paco.cftemplates import IAMRoles, IAMManagedPolicies,IAMUsers, IAMUserAccountDelegates, IAMSLRoles
 from paco.controllers.controllers import Controller
-from paco.core.exception import StackException
-from paco.core.exception import PacoErrorCode
-from paco.core.yaml import YAML, Ref, Sub
+from paco.core.exception import StackException, InvalidAccountPermission, PacoErrorCode
+from paco.core.yaml import YAML
 from paco.models.references import Reference
 from paco.models.locations import get_parent_by_interface
 from paco.models import schemas
 from paco.models.base import Named
-from paco.stack import StackOrder, Stack, StackGroup, StackTags, StackHooks
+from paco.stack import StackOrder, StackGroup, StackTags, StackHooks
 from paco.utils import md5sum, get_support_resource_ref_ext
 from parliament import analyze_policy_string
-import click
-import json
-import os
 import paco
-import time
-
 
 yaml=YAML(typ='safe')
 
@@ -121,29 +115,6 @@ class RoleContext():
     def get_aws_name(self):
         return self.aws_name()
 
-    def get_role(self, role_id=None, role_ref=None):
-        role_by_id = None
-        role_by_ref = None
-        if role_id !=  None:
-            for role in self.roles:
-                if role.id == role_id:
-                    role_by_id = role
-                    break
-        if role_ref != None:
-            role_by_ref = role[role_ref]
-
-        if role_by_id != None and role_by_ref != None:
-            if role_by_id.id == role_by_ref.id:
-                return role_by_id
-            else:
-                # You specified both role_id and role_ref
-                # but they each returned different results.
-                raise StackException(PacoErrorCode)
-        elif role_by_id != None:
-            return role_by_id
-
-        return role_by_ref
-
     def add_managed_policy(
         self,
         resource,
@@ -168,7 +139,7 @@ class RoleContext():
         paco.models.loader.apply_attributes_from_config(policy, policy_dict)
 
         if policy.paco_ref_parts in self.policy_context.keys():
-            print("Managed policy already exists: %s" % (policy_ref) )
+            print(f"Managed policy already exists: {policy.paco_ref_parts}")
             raise StackException(PacoErrorCode.Unknown)
 
         # set the resolve_ref_obj to this RoleContext
@@ -227,7 +198,7 @@ class IAMController(Controller):
         self.role_context = {}
         self.sl_role_context = {}
         self.policy_context = {}
-        self.iam_config = self.paco_ctx.project['resource']['iam']
+        self.iam = self.paco_ctx.project['resource']['iam']
         self.iam_user_stack_groups = {}
         self.iam_user_access_keys_sdb_domain = 'Paco-IAM-Users-Access-Keys-Meta'
         self.init_done = False
@@ -286,6 +257,40 @@ class IAMController(Controller):
             if permission_config not in permissions_by_account[account_name]:
                 permissions_by_account[account_name].append(permission_config)
 
+     # DeploymentPipelines
+    def init_deploymentpipelines_permission(self, permission_config, permissions_by_account):
+        """
+        Iterates over each pipeline reference and adds its permission config
+        to the map of permissions by account.
+        """
+        for resource in permission_config.resources:
+            pipeline_ref = Reference(resource.pipeline)
+            account_ref = 'paco.ref ' + '.'.join(pipeline_ref.parts) + '.configuration.account'
+            account_ref = self.paco_ctx.get_ref(account_ref)
+            account_name = self.paco_ctx.get_ref(account_ref + '.name')
+            if permission_config not in permissions_by_account[account_name]:
+                permissions_by_account[account_name].append(permission_config)
+
+
+            # Initialize The network environments that we need access into
+            pipeline_config = pipeline_ref.get_model_obj(self.paco_ctx.project)
+            self.paco_ctx.get_controller(pipeline_ref.parts[0], model_obj=pipeline_config)
+
+            # Some actions in the pipeline might be in different account so we must
+            # iterate the pipeline stages and actions and add them too.
+            for action_name in pipeline_config.source.keys():
+                action = pipeline_config.source[action_name]
+                account_name = None
+                if action.type == 'CodeDeploy.Deploy':
+                    asg_ref = Reference(action.auto_scaling_group)
+                    asg_config = asg_ref.get_model_obj(self.paco_ctx.project)
+                    account_name = self.paco_ctx.get_ref(asg_config.get_account().paco_ref + '.name')
+                if account_name != None:
+                    if permission_config not in permissions_by_account[account_name]:
+                        permissions_by_account[account_name].append(permission_config)
+
+
+
     def get_sdb_attribute_value(self, sdb_client, sdb_domain, item_name, attribute_name):
         attributes = sdb_client.get_attributes(
             DomainName=sdb_domain,
@@ -516,6 +521,7 @@ class IAMController(Controller):
         return cache_id
 
     def init_users(self, model_obj):
+        "Initialize IAM User StackGroups"
         self.stack_group_filter = model_obj.paco_ref_parts
         master_account_ctx = self.paco_ctx.get_account_context(account_ref='paco.ref accounts.master')
         # StackGroup
@@ -523,9 +529,9 @@ class IAMController(Controller):
             account_ctx = self.paco_ctx.get_account_context('paco.ref accounts.'+account_name)
             self.iam_user_stack_groups[account_name] = IAMUserStackGroup(self.paco_ctx, account_ctx, account_name, self)
 
-        stack_hooks = StackHooks(self.paco_ctx)
-        for user_name in self.iam_config.users.keys():
-            user_config = self.iam_config.users[user_name]
+        stack_hooks = StackHooks()
+        for user_name in self.iam.users.keys():
+            user_config = self.iam.users[user_name]
             # Stack hooks for managing access keys
             for hook_action in ['create', 'update']:
                 stack_hooks.add(
@@ -537,39 +543,54 @@ class IAMController(Controller):
                     hook_arg=user_config
                 )
 
+        # If account_whitelist is not ['all'] then validate that there are no accounts specified
+        # in permissions that are not part of the account_whitelist
+        for iam_user in self.iam.users.values():
+            allowed = {}
+            for account in iam_user.account_whitelist:
+                allowed[account] = None
+            # If all accounts are allowed, no need to do this check.
+            if 'all' in allowed:
+                break
+            for permission in iam_user.permissions.values():
+                # Some permissions don't have accounts so we ignore them here
+                if hasattr(permission, 'accounts') == False:
+                    continue
+                for account in permission.accounts:
+                    if account != 'all' and account not in allowed:
+                        raise InvalidAccountPermission(
+    f"User {iam_user.name} has permission {permission.name} for account {account} - that account is not granted in that user's account_whitelist."
+                        )
+
         # stack for the IAM User - this only exists in the Master account
         # and delegate roles are provisioned in accounts
         self.iam_user_stack_groups['master'].add_new_stack(
             master_account_ctx.config.region,
-            self.iam_config.users,
+            self.iam.users,
             IAMUsers,
             account_ctx=master_account_ctx,
             stack_hooks=stack_hooks,
         )
 
-        for user_name in self.iam_config.users.keys():
-            user_config = self.iam_config.users[user_name]
-
+        for user in self.iam.users.values():
             # Build a list of permissions for each account
             permissions_by_account = {}
             # Initialize permission_by_account for all accounts
             for account_name in self.paco_ctx.project['accounts'].keys():
                 permissions_by_account[account_name] = []
 
-            for permission_name in user_config.permissions.keys():
-                permission_config = user_config.permissions[permission_name]
+            for permission_name in user.permissions.keys():
+                permission_config = user.permissions[permission_name]
                 init_method = getattr(self, "init_{}_permission".format(permission_config.type.lower()))
                 init_method(permission_config, permissions_by_account)
 
             # Give access to accounts the user has explicit access to
-
             for account_name in self.paco_ctx.project['accounts'].keys():
-                account_ctx = self.paco_ctx.get_account_context('paco.ref accounts.'+account_name)
-                config_ref = 'resource.iam.users.'+user_name
+                account_ctx = self.paco_ctx.get_account_context('paco.ref accounts.' + account_name)
                 # IAM User delegate stack
                 self.iam_user_stack_groups[account_name].add_new_stack(
                     master_account_ctx.config.region,
-                    user_config,
+                    user,
                     IAMUserAccountDelegates,
                     stack_orders=[StackOrder.PROVISION ,StackOrder.WAITLAST],
                     account_ctx=account_ctx,
@@ -581,6 +602,7 @@ class IAMController(Controller):
                 )
 
     def init(self, command=None, model_obj=None):
+        "Initialize Controller's StackGroup for resource.iam scope"
         if model_obj == None:
             return
         if self.init_done == True:
