@@ -36,6 +36,8 @@ A generic placeholder for any schema.
 
 {IDeployable}
 
+{IEnablable}
+
 {IType}
 
 {IDNSEnablable}
@@ -46,12 +48,18 @@ A generic placeholder for any schema.
 
 {IRegionContainer}
 
+{IAccountRegions}
+
 {INotifiable}
 
 {ISecurityGroupRule}
 
 {IApplicationEngine}
 
+{IVPCConfiguration}
+
+{IHostedZone}
+
 Function
 --------
 
@@ -65,6 +73,10 @@ A callable function that returns a value.
 
 .. _resourcegroups: yaml-netenv.html#resourcegroups
 
+.. _securitygroup: yaml-netenv.html#securitygroup
+
+.. _segment: yaml-netenv.html#segment
+
 .. _metric: yaml-monitoring.html#metric
 
 .. _alarmnotifications: yaml-monitoring.html#alarmnotifications
@@ -92,6 +104,8 @@ defines one AWS account, the filename is the ``name`` of the account, with a .ym
 
 {IAccount}
 
+{IAdminIAMUsers}
+
 {IAdminIAMUser}
 
 """
@@ -105,32 +119,8 @@ Global Resources
 Global Resources are defined in the top-level ``resource/`` directory. They define cloud resources
 which do not belong to an environment or other logical grouping.
 
-CloudTrail
-----------
+{ICloudTrail}
 
-The ``resource/cloudtrail.yaml`` file contains CloudTrails.
-
-.. code-block:: bash
-
-    paco provision resource.cloudtrail
-
-
-.. code-block:: yaml
-    :caption: Example resource/cloudtrail.yaml file
-
-    trails:
-      cloudtrail:
-        region: ''
-        enabled: true
-        cloudwatchlogs_log_group:
-          expire_events_after_days: '14'
-          log_group_name: 'CloudTrail'
-        enable_log_file_validation: true
-        include_global_service_events: true
-        is_multi_region_trail: true
-        enable_kms_encryption: true
-        s3_bucket_account: 'paco.ref accounts.security'
-        s3_key_prefix: 'cloudtrails'
 
 CodeCommit
 ----------
@@ -272,14 +262,20 @@ IAM Users can be managed, see `Managing IAM Users with Paco`_.
 
 {IIAMUserPermissions}
 
+{IBaseRole}
+
 {IRole}
 
+{IRoleDefaultEnabled}
+
 {IAssumeRolePolicy}
 
 {IPolicy}
 
 {IStatement}
 
+{IPrincipal}
+
 Route 53
 --------
 
@@ -340,6 +336,12 @@ SNS has only two resources: SNS Topics and SNS Subscriptions.
     You will need this if you want to send CloudWatch Alarms from multiple accounts to the same
     SNS Topic(s) in one account.
 
+.. _s3bucket: yaml-app-resources.html#s3bucket
+
+.. _cloudwatchloggroup: yaml-app-resources.html#cloudwatchloggroup
+
+.. _vpc: yaml-netenv-resources.html#vpc
+
 """
 
 yaml_netenv = """
@@ -733,6 +735,10 @@ as part of an Application.
 
 {IApiGatewayMethodMethodResponseModel}
 
+{IApiGatewayCognitoAuthorizers}
+
+{IApiGatewayDNS}
+
 
 {IASG}
 
@@ -746,6 +752,12 @@ as part of an Application.
 
 {IASGRollingUpdatePolicy}
 
+{IECSASGConfiguration}
+
+{IECSCapacityProvider}
+
+{ISSHAccess}
+
 {IBlockDeviceMapping}
 
 {IBlockDevice}
@@ -791,7 +803,7 @@ as part of an Application.
 {ICloudFormationInitUsers}
 
 
-{IAWSCertificateManager}
+{IACM}
 
 
 {ICloudFront}
@@ -814,6 +826,10 @@ as part of an Application.
 
 {ICloudFrontCookies}
 
+{ICloudFrontLambdaFunctionAssocation}
+
+{ICognitoLambdaTriggers}
+
 
 {ICodeDeployApplication}
 
@@ -822,6 +838,30 @@ as part of an Application.
 {ICodeDeployDeploymentGroup}
 
 
+{ICognitoUserPool}
+
+{ICognitoInviteMessageTemplates}
+
+{ICognitoUserPoolClients}
+
+{ICognitoUserPoolClient}
+
+{ICognitoEmailConfiguration}
+
+{ICognitoUserPoolPasswordPolicy}
+
+{ICognitoUserPoolSchemaAttribute}
+
+{ICognitoUICustomizations}
+
+{ICognitoUserCreation}
+
+
+{ICognitoIdentityPool}
+
+{ICognitoIdentityProvider}
+
+
 {IDeploymentPipeline}
 
 {ICodePipelineStages}
@@ -836,6 +876,8 @@ as part of an Application.
 
 {IDeploymentPipelineDeployCodeDeploy}
 
+{IDeploymentPipelineSourceECR}
+
 {ICodeDeployMinimumHealthyHosts}
 
 {IDeploymentPipelineManualApproval}
@@ -859,6 +901,58 @@ as part of an Application.
 {IEC2}
 
 
+{IECRRepository}
+
+{IECRRepositoryPermission}
+
+{IECSCluster}
+
+
+{IECSServices}
+
+{IECSServicesContainer}
+
+{IECSService}
+
+{IECSTaskDefinitions}
+
+{IECSTaskDefinition}
+
+{IECSContainerDefinitions}
+
+{IECSContainerDefinition}
+
+{IECSLoadBalancer}
+
+{IECSVolume}
+
+{IECSUlimit}
+
+{IECSHealthCheck}
+
+{IECSHostEntry}
+
+{IDockerLabels}
+
+{IECSContainerDependency}
+
+{IECSTaskDefinitionSecret}
+
+{IECSLogging}
+
+{IECSVolumesFrom}
+
+{IECSTargetTrackingScalingPolicies}
+
+{IECSTargetTrackingScalingPolicy}
+
+{IServiceVPCConfiguration}
+
+{IECSMountPoint}
+
+{IPortMapping}
+
+
 {IEIP}
 
 
@@ -896,8 +990,14 @@ An unconstrainted set of key-value pairs used to set advanced options for Elasti
 
 {ILambdaVariable}
 
+{ILambdaAtEdgeConfiguration}
 
-{ILBApplication}
+
+{ILoadBalancer}
+
+{IApplicationLoadBalancer}
+
+{INetworkLoadBalancer}
 
 {IDNS}
 
@@ -914,21 +1014,69 @@ An unconstrainted set of key-value pairs used to set advanced options for Elasti
 {ITargetGroup}
 
 
+{IPinpointApplication}
+
+{IPinpointSMSChannel}
+
+{IPinpointEmailChannel}
+
+
+{IIoTTopicRule}
+
+{IIoTTopicRuleAction}
+
+{IIoTTopicRuleIoTAnalyticsAction}
+
+{IIoTTopicRuleLambdaAction}
+
+
+{IIoTAnalyticsPipeline}
+
+{IIoTDatasets}
+
+{IIoTDataset}
+
+{IDatasetTrigger}
+
+{IDatasetContentDeliveryRules}
+
+{IDatasetContentDeliveryRule}
+
+{IDatasetS3Destination}
+
+{IDatasetQueryAction}
+
+{IDatasetContainerAction}
+
+{IDatasetVariables}
+
+{IDatasetVariable}
+
+{IIoTPipelineActivities}
+
+{IIoTPipelineActivity}
+
+{IAttributes}
+
+{IIotAnalyticsStorage}
+
+{IStorageRetention}
+
+
+
 {IManagedPolicy}
 
 RDS
 ---
 
-Relational Database Service (RDS) is a collection of relational databases.
+Relational Database Service (RDS) allows you to set up, operate, and scale a relational database in AWS.
 
-There is no plain vanilla RDS type, but rather choose the type that specifies which kind of relational database
-engine to use. For example, ``RDSMysql`` for MySQL on RDS or ``RDSAurora`` for an Amazon Aurora database.
+You can create a single DB Instance or an Aurora DB Cluster.
 
-If you want to use DB Parameter Groups with your RDS, then use the ``parameter_group`` field to
-reference a DBParameterGroup_ resource. Keeping DB Parameter Group as a separate resource allows you
-to have multiple Paramater Groups provisioned at the same time. For example, you might have both
-resources for ``dbparams_performance`` and ``dbparams_debug``, allowing you to use the AWS
-Console to switch between performance and debug configuration quickl in an emergency.
+DB Instance
+^^^^^^^^^^^
+
+Currently Paco supports ``RDSMysql`` and ``RDSPostgresql`` for single database instances.
 
 .. sidebar:: Prescribed Automation
 
@@ -996,21 +1144,180 @@ Console to switch between performance and debug configuration quickl in an emerg
   primary_hosted_zone: paco.ref netenv.mynet.network.vpc.private_hosted_zone
   parameter_group: paco.ref netenv.mynet.applications.app.groups.web.resources.dbparams_performance
 
+Aurora DB Cluster
+^^^^^^^^^^^^^^^^^
+
+AWS Aurora is relational databases built for the cloud. Aurora features a distributed, fault-tolerant,
+self-healing storage system and can easily scale from a single database instance to a cluster of multiple
+database instances.
+
+When creating an Aurora RDS resource, you must specify your ``db_instances``. If you specify more than
+one database instance, then Aurora will automatically designate one instance as a Writer and all other
+instances will be Readers.
+
+Each ``db_instance`` can specify it's own complete set of configuration or you can use the ``default_instance``
+field to shared default configuration between instances. If a ``db_instance`` doesn't specify a value but it is
+specified by ``default_instance`` it will fall back to using that value.
+
+A simple Aurora with only a single database instance could be:
+
+.. code-block:: yaml
+  :caption: Simple Aurora single instance
+
+  type: RDSMysqlAurora
+  default_instance:
+    db_instance_type: db.t3.medium
+  db_instances:
+    single:
+
+A more complex Aurora with a cluster of three database instances could be:
+
+.. code-block:: yaml
+  :caption: Three instance Aurora cluster
+
+  type: RDSMysqlAurora
+  default_instance:
+    db_instance_type: db.t3.medium
+    enhanced_monitoring_interval_in_seconds: 30
+  db_instances:
+    first:
+      availability_zone: 1
+      db_instance_type: db.t3.large
+      enhanced_monitoring_interval_in_seconds: 5
+    second:
+      availability_zone: 2
+    third:
+      availability_zone: 3
+
+.. sidebar:: Prescribed Automation
+
+  ``secrets_password``: Uses a Secrets Manager secret for the database master password.
+
+  ``enable_kms_encryption``: Encrypts the database storage. Paco will creates a KMS-CMK dedicated to the
+  DB Cluster. This key can only be accessed by the AWS RDS service.
+
+  ``enhanced_monitoring_interval_in_seconds``: Paco will create an IAM Role to allow the RDS monitoring service
+  access to perform enhanced monitoring.
+
+  ``cluster_event_notifications`` and ``event_notifications`` must reference a group specified in ``resource/sns.yaml``.
+  This group (SNS Topic) must already be provisioned in the same account and region as the database.
+
+  ``monitoring`` applies to ``db_instances`` and will apply CloudWatch Alarms that are specific to each database instance
+  in the Aurora cluster.
+
+.. code-block:: yaml
+  :caption: RDSPostgresqlAurora db cluster example
+
+  type: RDSPostgresqlAurora
+  order: 10
+  enabled: true
+  availability_zones: all
+  engine_version: '11.7'
+  port: 5432
+  master_username: master
+  secrets_password: paco.ref netenv.anet.secrets_manager.anet.app.database
+  backup_preferred_window: 04:00-05:00
+  backup_retention_period: 7
+  maintenance_preferred_window: 'Sat:07:00-Sat:08:00'
+  cluster_parameter_group: paco.ref netenv.mynet.applications.app.groups.web.resources.clusterparams
+  cloudwatch_logs_exports:
+    - error
+  security_groups:
+    - paco.ref netenv.mynet.network.vpc.security_groups.app.database
+  segment: paco.ref netenv.anet.network.vpc.segments.private
+  dns:
+    - domain_name: database.test.internal
+      hosted_zone: paco.ref netenv.mynet.network.vpc.private_hosted_zone
+  enable_kms_encryption: true
+  cluster_event_notifications:
+    groups:
+      - wb_low
+    event_categories:
+      - failover
+      - failure
+      - notification
+  default_instance:
+    parameter_group: paco.ref netenv.mynet.applications.app.groups.web.resources.dbparams_performance
+    enable_performance_insights: true
+    publicly_accessible: false
+    db_instance_type: db.t3.medium
+    allow_major_version_upgrade: true
+    auto_minor_version_upgrade: true
+    event_notifications:
+      groups:
+        - admin
+      event_categories:
+        - availability
+        - configuration change
+        - deletion
+        - failover
+        - failure
+        - maintenance
+        - notification
+        - recovery
+    monitoring:
+      enabled: true
+      alarm_sets:
+        basic_dbinstance:
+  db_instances:
+    first:
+      db_instance_type: db.t3.medium
+      enhanced_monitoring_interval_in_seconds: 30
+      availability_zone: 1
+      monitoring:
+        enabled: true
+        alarm_sets:
+          complex_dbinstance:
+    second:
+      enable_performance_insights: false
+      event_notifications:
+        groups:
+          - admin
+        event_categories:
+          - maintenance
+
+
+{IRDSMysql}
+
+{IRDSPostgresql}
+
+{IRDSPostgresqlAurora}
+
+{IRDSMysqlAurora}
 
 {IRDSOptionConfiguration}
 
 {INameValuePair}
 
-{IRDSMysql}
+{IRDSMultiAZ}
+
+{IRDSInstance}
 
 {IRDSAurora}
 
-{IDBParameterGroup}
+{IRDSDBInstanceEventNotifications}
+
+{IRDSClusterDefaultInstance}
+
+{IRDSClusterInstance}
+
+{IRDSClusterInstances}
+
+{IRDSDBClusterEventNotifications}
 
 DBParameters
 ^^^^^^^^^^^^
 
-A unconstrainted set of key-value pairs.
+If you want to use DB Parameter Groups with your RDS, then use the ``parameter_group`` field to
+reference a DBParameterGroup_ resource. Keeping DB Parameter Groups as separate resources allows
+having multiple Paramater Groups provisioned at the same time. For example, you might have both
+resources for ``dbparams_performance`` and ``dbparams_debug``, allowing you to use the AWS
+Console to switch between performance and debug configuration quickl in an emergency.
+
+{IDBParameterGroup}
+
+{IDBClusterParameterGroup}
+
 
 {IRoute53HealthCheck}
 
@@ -1023,6 +1330,9 @@ A unconstrainted set of key-value pairs.
 
 {IS3NotificationConfiguration}
 
+{IS3StaticWebsiteHosting}
+
+{IS3StaticWebsiteHostingRedirectRequests}
 
 {ISNSTopic}
 
@@ -1031,6 +1341,8 @@ A unconstrainted set of key-value pairs.
 
 .. _role: yaml-global-resources.html#role
 
+.. _roledefaultenabled: yaml-global-resources.html#roledefaultenabled
+
 .. _ec2keypair: yaml-global-resources.html#ec2keypair
 
 .. _secretsmanagersecret: yaml-netenv#secretsmanagersecret
@@ -1045,12 +1357,19 @@ A unconstrainted set of key-value pairs.
 
 .. _codecommitrepository: yaml-global-resources.html#codecommitrepository
 
+.. _codecommituser: yaml-global-resources.html#codecommituser
+
 .. _segment: yaml-netenv.html#segment
 
 .. _cloudwatchlogretention: yaml-monitoring.html#cloudwatchlogretention
 
 .. _statement: yaml-global-resources.html#statement
 
+.. _vpcconfiguration: yaml-base.html#vpcconfiguration
+
+.. _hostedzone: yaml-base.html#hostedzone
+
+
 """
 
 
@@ -1128,6 +1447,7 @@ def convert_schema_to_list_table(schema, level='-', header=True):
     Introspects a Schema-based Interface and returns
     a ReStructured Text representation of it.
     """
+    table_row_template = ''
     if schema.__name__ in ('IFunction', 'IDBParameters'):
         return ''
     schema_name = strip_interface_char(schema.__name__)
@@ -1268,7 +1588,9 @@ def convert_field_to_table_row(schema, field, table_row_template):
         else:
             data_type = 'Dict'
     elif data_type == 'List':
-        if field.value_type and not zope.schema.interfaces.IText.providedBy(field.value_type):
+        if field.value_type and zope.schema.interfaces.IChoice.providedBy(field.value_type):
+            data_type = 'Choice'
+        elif field.value_type and not zope.schema.interfaces.IText.providedBy(field.value_type):
             data_type = 'List<{}_>'.format(
                 strip_interface_char(field.value_type.schema.__name__)
             )
@@ -1346,10 +1668,17 @@ MINOR_SCHEMAS = {
     'IApiGatewayMethodMethodResponseModel': None,
     'IApiGatewayMethodIntegration': None,
     'IApiGatewayMethodIntegrationResponse': None,
+    'IApiGatewayCognitoAuthorizers': None,
+    'IApiGatewayDNS': None,
     'IDNS': None,
     'IASGLifecycleHook': None,
     'IASGScalingPolicy': None,
     'IASGRollingUpdatePolicy': None,
+    'IECSASGConfiguration': None,
+    'IECSCapacityProvider': None,
+    'IECSTargetTrackingScalingPolicies': None,
+    'IECSTargetTrackingScalingPolicy': None,
+    'ISSHAccess': None,
     'IListener': None,
     'ITargetGroup': None,
     'IListeners': None,
@@ -1368,6 +1697,7 @@ MINOR_SCHEMAS = {
     'IAssumeRolePolicy': None,
     'IPolicy': None,
     'IStatement': None,
+    'IPrincipal': None,
     'IIAMUser': None,
     'IIAMUserProgrammaticAccess': None,
     'IIAMUserPermissions': None,
@@ -1380,11 +1710,14 @@ MINOR_SCHEMAS = {
     'ILambdaEnvironment': None,
     'ILambdaVpcConfig': None,
     'ILambdaVariable': None,
+    'ILambdaAtEdgeConfiguration': None,
     'IS3BucketPolicy': None,
     'IS3StaticWebsiteHosting': None,
     'IS3StaticWebsiteHostingRedirectRequests': None,
     'IS3LambdaConfiguration': None,
     'IS3NotificationConfiguration': None,
+    'IS3StaticWebsiteHosting': None,
+    'IS3StaticWebsiteHostingRedirectRequests': None,
     'ISNSTopicSubscription': None,
     'ICloudFrontCacheBehavior': None,
     'ICloudFrontFactory': None,
@@ -1396,13 +1729,18 @@ MINOR_SCHEMAS = {
     'ICloudFrontDefaultCacheBehavior': None,
     'ICloudFrontForwardedValues': None,
     'ICloudFrontCookies': None,
+    'ICloudFrontLambdaFunctionAssocation': None,
     'ICloudFrontDefaultCacheBehavior': None,
+    'ICognitoLambdaTriggers': None,
+    'ICloudFrontOrigins': None,
+    'ICloudFrontFactories': None,
     'ICodePipelineStages': None,
     'ICodePipelineStage': None,
     'IDeploymentPipelineStageAction': None,
     'IDeploymentPipelineSourceStage': None,
     'IDeploymentPipelineDeployStage': None,
     'IDeploymentPipelineBuildStage': None,
+    'IDeploymentPipelineSourceECR': None,
     'IDeploymentPipelineDeployCodeDeploy': None,
     'ICodeDeployMinimumHealthyHosts': None,
     'IDeploymentPipelineManualApproval': None,
@@ -1412,9 +1750,39 @@ MINOR_SCHEMAS = {
     'IDeploymentPipelineConfiguration': None,
     'IDeploymentGroupS3Location': None,
     'IRDSMysql': None,
+    'IRDSMultiAZ': None,
+    'IRDSInstance': None,
+    'IRDSDBInstanceEventNotifications': None,
+    'IRDSClusterDefaultInstance': None,
+    'IRDSClusterInstance': None,
+    'IRDSClusterInstances': None,
+    'IRDSDBClusterEventNotifications': None,
+    'IDBClusterParameterGroup': None,
+    'IRDSMysqlAurora': None,
+    'IRDSPostgresql': None,
+    'IRDSPostgresqlAurora': None,
     'IRDSAurora': None,
     'IElastiCacheRedis': None,
     'IElasticsearchCluster': None,
+    'IECSServicesContainer': None,
+    'IECSService': None,
+    'IECSTaskDefinitions': None,
+    'IECSTaskDefinition': None,
+    'IECSContainerDefinitions': None,
+    'IECSContainerDefinition': None,
+    'IECSLoadBalancer': None,
+    'IECSVolume': None,
+    'IECSUlimit': None,
+    'IECSHealthCheck': None,
+    'IECSHostEntry': None,
+    'IDockerLabels': None,
+    'IECSContainerDependency': None,
+    'IECSTaskDefinitionSecret': None,
+    'IECSLogging': None,
+    'IECSVolumesFrom': None,
+    'IECSMountPoint': None,
+    'IServiceVPCConfiguration': None,
+    'IPortMapping': None,
     'IEBSOptions': None,
     'IESAdvancedOptions': None,
     'ICodeDeployDeploymentGroups': None,
@@ -1424,7 +1792,9 @@ MINOR_SCHEMAS = {
     'IIAMUser': None,
     'IIAMUserProgrammaticAccess': None,
     'IIAMUserPermissions': None,
+    'IBaseRole': None,
     'IRole': None,
+    'IRoleDefaultEnabled': None,
     'IAssumeRolePolicy': None,
     'IPolicy': None,
     'IStatement': None,
@@ -1527,6 +1897,36 @@ MINOR_SCHEMAS = {
     'IBackupPlanRule': None,
     'IBackupPlanSelection': None,
     'IBackupSelectionConditionResourceType': None,
+    'IIoTTopicRuleAction': None,
+    'IIoTTopicRuleIoTAnalyticsAction': None,
+    'IIoTTopicRuleLambdaAction': None,
+    'IIoTDatasets': None,
+    'IIoTDataset': None,
+    'IDatasetTrigger': None,
+    'IDatasetContentDeliveryRules': None,
+    'IDatasetContentDeliveryRule': None,
+    'IDatasetS3Destination': None,
+    'IDatasetQueryAction': None,
+    'IDatasetContainerAction': None,
+    'IDatasetVariables': None,
+    'IDatasetVariable': None,
+    'IIoTPipelineActivities': None,
+    'IIoTPipelineActivity': None,
+    'IAttributes': None,
+    'IIotAnalyticsStorage': None,
+    'IStorageRetention': None,
+    'IPinpointSMSChannel': None,
+    'IPinpointEmailChannel': None,
+    'ICognitoUserPoolClients': None,
+    'ICognitoInviteMessageTemplates': None,
+    'ICognitoUserPoolClient': None,
+    'ICognitoEmailConfiguration': None,
+    'ICognitoUserPoolPasswordPolicy': None,
+    'ICognitoUserPoolSchemaAttribute': None,
+    'ICognitoUICustomizations': None,
+    'ICognitoUserCreation': None,
+    'ICognitoIdentityProvider': None,
+    'IECRRepositoryPermission': None,
 }
 
 def create_tables_from_schema():
@@ -1561,6 +1961,8 @@ yaml_base_links = """
 
 .. _Deployable: yaml-base.html#Deployable
 
+.. _Enablable: yaml-base.html#Enablable
+
 .. _SecurityGroupRule: yaml-base.html#SecurityGroupRule
 
 .. _ApplicationEngine: yaml-base.html#ApplicationEngine
@@ -1573,6 +1975,8 @@ yaml_base_links = """
 
 .. _resource: yaml-base.html#resource
 
+.. _accountregions: yaml-base#accountregions
+
 .. _type: yaml-base.html#type
 
 .. _interface: yaml-base.html#interface
